/****************************************************************************
 *
 *                               IBM INC.
 *
 *                           PROPRIETARY DATA
 *
 * Licensed Material - Property Of IBM
 *
 * "Restricted Materails of IBM"
 *
 * IBM Informix Client SDK
 *
 * (c)  Copyright IBM Corporation 2002. All rights reserved.
 *
 ****************************************************************************
 */


//  Title: simpval.cpp
//
//  Description:
//   Simple value object example

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#include <it.h>
#include <string.h>
// Windows header has different name
#ifndef IT_WINDOWS
#include <strstream.h>
#else
#include <strstrea.h>
#endif

//
//  Bitarray -- value object for bit arrays
//
//      This interface provides the ability to print bit array
//      datatypes to strings. It is derived directly from the ITValue 
//      class,which means it can support only that interface.
//

#define NBITS 32

typedef mi_integer bitarray_t;                                      // 1 begin
                                                                    // 1 end
class Bitarray : public ITDatum                                     // 2 begin
{
public:
    
    // Overrides of ITEssential methods
    virtual ITOpErrorCode IT_STDCALL QueryInterface(const ITInterfaceID &ifiid,
                                                    void **resultif);
    virtual unsigned long IT_STDCALL AddRef();
    virtual unsigned long IT_STDCALL Release();

    // Overrides of ITValue methods
    virtual const ITString & IT_STDCALL Printable();
    virtual const ITTypeInfo & IT_STDCALL TypeOf();
    virtual ITBool IT_STDCALL IsNull();
    virtual ITBool IT_STDCALL SameType(ITValue *);
    virtual ITBool IT_STDCALL CompatibleType(ITValue *);
    virtual ITBool IT_STDCALL Equal(ITValue *);
    virtual ITBool IT_STDCALL LessThan(ITValue *);
    virtual ITBool IT_STDCALL IsUpdated();
    virtual ITBool IT_STDCALL FromPrintable( const ITString & );
    virtual ITBool IT_STDCALL SetNull();

    // Overrides of ITDatum methods
    virtual MI_DATUM IT_STDCALL Data();
    virtual long IT_STDCALL DataLength();
    virtual ITBool IT_STDCALL SetData( MI_DATUM, long, ITPreserveData * );
    virtual const ITConnection & IT_STDCALL Connection();
  
    // Class constructor, destructor
    Bitarray(ITMVDesc *);
    ~Bitarray();
    
    // Factory Constructor -- this is the entry point for objects to
    // be created. It uses the class constructor to build an object
    // and returns in to the caller. It is called automatically by the
    // Interface when an object of the "bitarray" type is returned by
    // the server to the interface
    static ITValue *MakeValue(ITMVDesc *);

    // Data members to implement ITEssential functionality
    long refcount;

    // Data members to implement ITValue functionality
    ITTypeInfo typeinfo;
    ITBool isnull, isupdated;
    ITString printable_value;

    // Data members to implement bitarray storage
    bitarray_t  value;

    ITConnection conn;
};                                                                  // 2 end


// Implementation
// Value objects should be created with a refcount of 1, so the caller
// receives an object with an already-valid reference count.
Bitarray::Bitarray(ITMVDesc *mv)                                    // 3 begin
    : refcount(1),
      typeinfo(*mv->vf_origtypeinfo),
      isupdated( FALSE ),
      conn( *mv->vf_connection )
{
    // NULL?
    isnull = mv->vf_libmivaluetype == MI_NULL_VALUE;
    if( !isnull )
        value = *(bitarray_t *)mv->vf_data;
}                                                                   // 3 end

Bitarray::~Bitarray()
{
}

// Factory
// This global variable is defined only for its side-effect of adding
// the mapping
//
//  ("bitarray" server type) -> (Bitarray::MakeValue) 
//
// to the global type mapping list. By defining this globally, it is
// called before main() is started, which builds the object without
// any explicit initialization being required.

ITFactoryList BitarrayFactory("bitarray",                           // 4 begin
                              Bitarray::MakeValue);                 // 4 end

// Entry point for factory system
ITValue *                                                           // 5 begin
Bitarray::MakeValue(ITMVDesc *mv)
{
    return new Bitarray(mv);
}                                                                   // 5 end


// Override implementations

// This is a standard implementation of QueryInterface(). Note that
// since it returns a new reference, it increments the reference count
// if the result was successful.
ITOpErrorCode                                                       // 6 begin
Bitarray::QueryInterface(const ITInterfaceID &iid,
                                void **ifptr)
{
    int result = IT_QUERYINTERFACE_SUCCESS;
    
    switch (ITIIDtoSID(iid))
    {
        case ITEssentialSID:
        case ITValueSID:
        case ITDatumSID:
            *ifptr = this;
            break;
        
        default:
            result = IT_QUERYINTERFACE_FAILED;
            *ifptr = NULL;
            break;
    }
    if (result == IT_QUERYINTERFACE_SUCCESS)
        AddRef();

    return result;
}                                                                   // 6 end

// Standard implementations of AddRef and Release. Note that release
// destroys the object when the reference count goes to zero. In a
// more elaborate implementation, this could return the object for
// later use into a pool of objects of a specific class.
unsigned long
Bitarray::AddRef()
{
    return ++refcount;
}

unsigned long
Bitarray::Release()
{
    if (--refcount <= 0)
    {
        delete this;
        return 0;
    }
    else
    {
        return refcount;
    }
}

// Overrides of ITValue methods
// Returns the contents of bit array.
const ITString &                                                    // 7 begin
Bitarray::Printable()
{
    if( IsNull() )
        return printable_value = "null";

    char buf[32];
    ostrstream cstream( buf, sizeof buf );
    cstream << value << ends;
    return printable_value = cstream.str();
}                                                                   // 7 end

ITBool 
Bitarray::FromPrintable( const ITString &val )
{
    if( val.IsNull() )
        isnull = TRUE;
    else
        {
        istrstream cstream( (char *)val.Data(), val.Size() );
        cstream >> value;
        isnull = FALSE;
        }
    return isupdated = TRUE;
}

// Fairly generic implementations of ITValue calls
const ITTypeInfo &
Bitarray::TypeOf()
{
    return typeinfo;
}

ITBool
Bitarray::IsNull()
{
    return isnull;
}

ITBool
Bitarray::SameType(ITValue *v)
{
    return TypeOf().Name().Equal(v->TypeOf().Name());
}

// "Compatible" is a fairly loose concept of type comparison. For this
// object's implementation, compatible is defined to mean that it
// supports an ITValue interface as well, which is always true.
ITBool
Bitarray::CompatibleType(ITValue *)
{
    return TRUE;
}

ITBool
Bitarray::Equal(ITValue *v)
{
    // NULL is not equal to anything
    if( IsNull() || v->IsNull() )
        return FALSE;
    return Printable().Equal(v->Printable());
}

// Unsupported in this definition.
ITBool
Bitarray::LessThan(ITValue *)
{
    return FALSE;
}

ITBool 
Bitarray::IsUpdated()
{
    return isupdated;
}
 
ITBool 
Bitarray::SetNull()
{
    return isupdated = isnull = TRUE;
}

// ITDatum implementations
 
MI_DATUM
Bitarray::Data()
{
    return &value;
}
 
long
Bitarray::DataLength()
{
    return sizeof(bitarray_t);
}
 
ITBool
Bitarray::SetData(MI_DATUM data, long datalen, ITPreserveData *preservedata)
{
    value = *((bitarray_t *) data);
    isnull = FALSE;
    return isupdated = TRUE;
}
 
const ITConnection&
Bitarray::Connection()
{
    return conn;
}
